# Copyright 2015 OpenStack Foundation
# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.api.definitions import bgpvpn

BGPVPN_RES = bgpvpn.BGPVPN_RES
BGPVPN_L3 = bgpvpn.BGPVPN_L3
BGPVPN_L2 = bgpvpn.BGPVPN_L2
BGPVPN_TYPES = bgpvpn.BGPVPN_TYPES
UINT32_REGEX, = bgpvpn.UINT32_REGEX,
UINT16_REGEX = bgpvpn.UINT16_REGEX
UINT8_REGEX = bgpvpn.UINT8_REGEX
IP4_REGEX = bgpvpn.IP4_REGEX
RTRD_REGEX = bgpvpn.RTRD_REGEX
