{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body %}
<div class="left">
    <fieldset>
        {% include "horizon/common/_form_fields.html" %}
    </fieldset>
</div>
<div class="right">
    <h3>{% trans "Description:" %}</h3>
    <p>{% trans "Create a new BGP VPN for any project as you need."%}</p>
</div>
{% endblock %}