{% load i18n  %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "BGPVPN Name" %}</dt>
    <dd>{{ bgpvpn.name }}</dd>
    <dt>{% trans "BGPVPN ID" %}</dt>
    <dd>{{ bgpvpn.id }}</dd>
    <dt>{% trans "Type" %}</dt>
    <dd>{{ bgpvpn.type }}</dd>
    {% include "project/bgpvpn/_associated_networks.html" %}
    {% include "project/bgpvpn/_associated_routers.html" %}
  </dl>
</div>
